// $Id: Bib1Exception.java,v 1.2 2002/12/14 00:17:49 mike Exp $

package org.z3950.zoom;

/**
 * Represents an error occurring in a Z39.50 Server.
 * <P>
 * <B>Synopsis:</B>
 * <P>
 * <PRE>
 *	try {
 *	    // deep sequence of calls, one of which does:
 *		throw new Bib1Exception(238, "xml,sutrs");
 *	} catch (Bib1Exception ex) {
 *	    System.err.println("code=" + ex.code());
 *	    System.err.println("addinfo=" + ex.addinfo());
 *	}
 * </PRE>
 * <P>
 * This class is not described in detail in the ZOOM Abstract API, but
 * is hinted at in the discussion of diagnostics for the Connection
 * class.  It provides methods for obtaining the diagnostic details
 * returned from the server.
 *
 * @version	$Id: Bib1Exception.java,v 1.2 2002/12/14 00:17:49 mike Exp $
 * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.5"
 *	               >zoom.z3950.org/api/zoom-1.1.html#3.2.5</A>
 */
public class Bib1Exception extends ZOOMException {
    int errcode;

    Bib1Exception(int errcode, String addinfo) {
	super(addinfo);
	this.errcode = errcode;
    }

    /**
     * Returns the BIB1 diagnostic code of the error that occurred on
     * the server.  This is an integer whose interpretation is
     * specified in appendix ERR.1 (<I>Bib-1 Diagnostics</I>) of the
     * Z39.50 standard itself.
     *
     * @return		the BIB1 diagnostic code.
     * @see <A href="http://lcweb.loc.gov/z3950/agency/defns/bib1diag.html"
     *		>Z39.50-1995, Appendix ERR.1</A>
     * @see <A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.5"
     *		           >zoom.z3950.org/api/zoom-1.1.html#3.2.5</A>
     */
    public int errcode() {
	return errcode;
    }

    /**
     * Returns a human-readable string corresponding to the BIB1
     * diagnostic code of the server error as returned by the
     * <CODE>errcode()</CODE> method.
     *
     * @return		a string describing the BIB1 diagnostic code.
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.5"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.5</A>
     */
    public String errmsg() {
	return Global.code2msg(errcode);
    }

    /**
     * Returns any additional information provided by the server
     * concerning the error that is specified by the
     * <CODE>errcode()</CODE> method.  Its exact interpretation is
     * dependent on what that diagnostic code is.
     *
     * @return		additional information about the server error.
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.5"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.5</A>
     */
    public String addinfo() {
	return getMessage();
    }

    public String toString() {
	return ("Bib1Exception " + errcode() +
		" (" + errmsg() + "): " + addinfo());
    }
}
