// $Id: CCLQuery.java,v 1.3 2002/12/14 00:17:49 mike Exp $

package org.z3950.zoom;
import java.io.IOException;
import com.k_int.IR.IRQuery;
import com.k_int.IR.QueryModels.CCLString;

/**
 * Represents a CCL query to be submitted to a Z39.50 Server.
 * <P>
 * <B>Synopsis:</B>
 * <P>
 * <PRE>
 *	Query q = new CCLQuery("ti=minerals and (ab=location or ab=place)",
 *	                       "qualset.properties");
 *	ResultSet rs = conn.search(q);
 * </PRE>
 * <P>
 * Queries in ZOOM may take various forms: objects of this class
 * represent queries expressed in Common Command Language (CCL), as
 * described in ISO 8777, and implemented in the Europagate CCL
 * compiler.  I'd like to include a reference to the standard, but in
 * defiance of all rationality, ISO keeps its standards secret
 * (presumably out of fear that someone might implement them), so the
 * best I can offer is a reference to the CCL description that comes
 * with the Yaz toolkit.
 * <P>
 * As with all ZOOM Query types, there are no methods apart from the
 * constructor: the only thing you can do with a CCLQuery is feed it
 * to <CODE>Connection.search()</CODE>.
 *
 * @version	$Id: CCLQuery.java,v 1.3 2002/12/14 00:17:49 mike Exp $
 * @see	<A href="http://www.indexdata.dk/yaz/doc/tools.php#CCL"
 *	>The Yaz toolkit's CCL documentation</A>
 * @see <A href="http://www.acm.org/tsc/acmfree.htm"
 *	>Standards: Free or Sold?  Comm. ACM, 38, 2 pp 23-27</A>
 * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.3"
 *	>zoom.z3950.org/api/zoom-1.1.html#3.3</A>
 */
public class CCLQuery extends Query {
    /**
     * Creates a new Query object representing the specified CCL
     * query, with qualifiers interpreted according to the specified
     * qualifier set.
     * <P>
     * In CCL, search terms may be preceded by <I>qualifiers</I> such
     * as <CODE>au</CODE> (typically interpreted to mean search for an
     * author's name) followed by an equals sign: for example, the CCL
     * search <CODE>ti=park</CODE> might mean to search for books with
     * the word ``park'' in their title.
     * <P>
     * The qualifier set is a bundle of configuration information
     * which specifies how these CCL qualifiers should be mapped into
     * a form that can be expressed in a native Z39.50 query
     * structure.
     *
     * @param ccl	The query itself expressed in CCL.
     * @param qualset	A reference to a qualifier set, in some form
     *			yet to be specified: perhaps the name of a
     *			Java properties file?
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.3.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.3.2</A>
     */
    public CCLQuery(String ccl, String qualset) {
	//  ###	The `qualset' parameter is currently not used: JZKit's
	//	CCLString class is hardwired to use the default
	//	CCLConfig object, which is hardwired to take its
	//	information from a "ccl_defaults.xml" file on the
	//	classpath.  Ian, please add an optional parameter!
	q = new IRQuery();

	//  ###	This goes badly, badly wrong if you feed it something
	//	it doesn't like, such as "@attr 1=4 mineral" (which is
	//	course a PQF query, and not CCL).  It looks like
	//	JZKit's CCL parser goes into a tight loop until the
	//	TimeoutExceededException.  Don't Do That Then.
	q.query = new CCLString(ccl);
    }
}
