// $Id: Connection.java,v 1.3 2002/12/14 00:17:49 mike Exp $

package org.z3950.zoom;
import java.util.Hashtable;
import java.util.Properties;
import com.k_int.IR.Searchable;
import com.k_int.z3950.IRClient.Z3950Origin;

/**
 * Represents a connection to a Z39.50 Server, as described in the
 * ZOOM Abstract API.
 * <P>
 * <B>Synopsis:</B>
 * <P>
 * <PRE>
 *	Connection conn = new Connection("muffin.indexdata.dk", 210);
 *	conn.option("databaseName", "gils");
 *	conn.option("preferredRecordSyntax", "xml");
 * </PRE>
 *
 * @version	$Id: Connection.java,v 1.3 2002/12/14 00:17:49 mike Exp $
 * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2"
 *		       >zoom.z3950.org/api/zoom-1.1.html#3.2</A>
 */
public class Connection {
    private Hashtable options = new Hashtable();
    Searchable s; // Or should we admit it's a Z3950Origin?

    /**
     * Returns the value of the specified option in this Connection
     * object, if it is defined there; otherwise returns the global
     * option of that name.
     *
     * @param key	the name of the requested option
     * @return		the value of the specified option, or null if
     *			no such option has been defined.
     * @see		Global#option(String)
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.3"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.3</A>
     */
    public String option(String key) {
	String res = (String) options.get(key);
	if (res == null) res = Global.option(key);
	return res;
    }

    /**
     * Sets the specified option in this Connection object to the
     * specified value, returning its previous value (as would have
     * been returned by the single-parameter version.)
     *
     * @param key	the name of the option to set
     * @param value	the value to which to set the specified option
     * @return		the old value of the specified option, if any
     * @see		#option(String)
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.3"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.3</A>
     */
    public String option(String key, String val) {
	String old = option(key);
	options.put(key, val);
	return old;
    }

    /**
     * Creates a new connection object, and forges a Z39.50
     * connection to the server specified by the parameters.
     *
     * @param hostname	the name or IP address of the Z39.50 server
     * @param value	the IP port on which the server is listening
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.2</A>
     */
    public Connection(String hostname, int portnum) {
	Properties p = new Properties();
	p.put("ServiceHost", hostname);
	p.put("ServicePort", String.valueOf(portnum));

	s = new Z3950Origin();
	s.init(p);
	//  ###	This doesn't actually forge the connection until a
	//	search or other operation is submitted.  I'm not sure
	//	there's any proper way around that: reading the JZKit
	//	source suggests that checkConnection() would do it for
	//	us, but it's private.  A further problem is that, in
	//	the current JZkit (Fri Dec 13 23:17:52 GMT 2002), this
	//	doesn't actually tell you if it fails!
    }

    /**
     * Performs a search on the server to which this Connection is
     * attached, using the specified query, and returns a new Result
     * Set object.
     *
     * @param q		A Z39.50 Query expressed as one of the
     *			subtypes of the Query class, which see.
     * @return		A newly created ResultSet object representing
     *			the set of records which the server selected
     *			as satisfying the query.  This is the only way
     *			to create a ResultSet.
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.4"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.4</A>
     */
    public ResultSet search(Query q)
	throws SearchException, Bib1Exception {
	ResultSet rs = new ResultSet(this, q);
	rs._search();
	return rs;
    }

    /**
     * Closes the connection to the server, allowing it to release any
     * resources it has allocated to this client.
     *
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.2.6"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.2.6</A>
     */
    public void close() {
	s.destroy();
	s = null;
    }

    Searchable _JZConn() { return s; }
}
