// $Id: ResultSet.java,v 1.6 2002/12/14 00:17:49 mike Exp $

package org.z3950.zoom;
import java.util.Hashtable;
import java.util.Vector;
//import com.k_int.IR.*;
import com.k_int.IR.IRQuery;
import com.k_int.IR.SearchTask;
import com.k_int.IR.InformationFragmentSource;
import com.k_int.IR.InformationFragment;
import com.k_int.IR.PresentException;
import com.k_int.IR.RecordFormatSpecification;
import com.k_int.z3950.IRClient.Z3950Origin; // for property names

/**
 * Represents a result set held on a Z39.50 Server, as described in
 * the ZOOM Abstract API.
 * <P>
 * <B>Synopsis:</B>
 * <P>
 * <PRE>
 *	ResultSet rs = conn.search(query);
 *	int size = rs.size();
 *	for (int i = 0; i < size; i++) {
 *	    System.out.println(rs.getRecord(i).render());
 *	}
 * </PRE>
 * <P>
 * ResultSets are created only by the Connection class's
 * <CODE>search()</CODE> method, so there is no public constructor for
 * this class.
 *
 * @version	$Id: ResultSet.java,v 1.6 2002/12/14 00:17:49 mike Exp $
 * @see		Connection#search(Query)
 * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4"
 *		       >zoom.z3950.org/api/zoom-1.1.html#3.4</A>
 */
public class ResultSet {
    Hashtable options = new Hashtable();
    private Connection conn;
    private Query q;
    private SearchTask st;	// could be local to _search()
    private InformationFragmentSource ifs;

    /**
     * Returns the value of the specified option in this ResultSet
     * object, if it is defined there; otherwise returns the specified
     * option's value in the Connection object by which this ResultSet
     * was created.
     *
     * @param key	the name of the requested option
     * @return		the value of the specified option, or null if
     *			no such option has been defined.
     * @see		Connection#option(String)
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.2</A>
     */
    public String option(String key) {
	String res = (String) options.get(key);
	if (res == null) res = conn.option(key);
	return res;
    }

    /**
     * Sets the specified option in this ResultSet object to the
     * specified value, returning its previous value (as would have
     * been returned by the single-parameter version.)
     *
     * @param key	the name of the option to set
     * @param value	the value to which to set the specified option
     * @return		the old value of the specified option, if any
     * @see		#option(String)
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.2</A>
     */
    public String option(String key, String val) {
	String old = option(key);
	options.put(key, val);
	return old;
    }

    // package visibility: use only by Connection.search
    ResultSet(Connection conn, Query q) {
	this.conn = conn;
	this.q = q;
	// Nothing to actually do here: _search() does the work
    }

    // package visibility: used only by Connection.search
    void _search() throws SearchException, Bib1Exception {
	IRQuery irq = q._JZQuery();

	String dbname = conn.option("databaseName");
	if (dbname == null) {
	    // 23 -> Specified combination of databases not supported
	    throw new Bib1Exception(23, "no database specified");
	}

	Vector dbnames = new Vector();
	dbnames.add(dbname);
	// I'm not ecstatic about this modal messing with `irq'.
	// Maybe we should make a clone of `irq' and tweak that
	// instead?  It doesn't really matter since we know this
	// method is only ever called for a newly-created ResultSet,
	// so there's no possibiliy of thread-unsafeness; but still.
	irq.collections = dbnames;

	st = conn._JZConn().createTask(irq, null);
	try {
	    st.evaluate(60000);
	} catch (com.k_int.IR.SearchException e) {
	    // Transform this substrate-specific exception into a
	    // generic ZOOM one that will also be supported by other
	    // implementations.
	    throw new SearchException(e.getMessage());
	} catch (com.k_int.IR.TimeoutExceededException e) {
	    // BIB-1 diagnostic 31 is "Resources exhausted - no
	    // results available": not perfect, but it will do.
	    throw new Bib1Exception(31, "timeout exceeded");
	}

	ifs = st.getTaskResultSet();
    }

    /**
     * Returns the number of records found by the server satisfying
     * the query with which this ResultSet was created.
     *
     * @return		the number of records in the ResultSet
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.3"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.3</A>
     */
    public int size() {
	return ifs.getFragmentCount();
    }

    /**
     * Fetches the <CODE>i</CODE>th record of this ResultSet from the
     * server on the other end of the Connection with which the
     * ResultSet was created.  Counting starts from zero, so that the
     * highest acceptable value of <CODE>i</CODE> is one less than the
     * return value of the <CODE>size()</CODE> method.
     *
     * @param i		zero-based index of the record to fetch.
     * @return		a newly created Record object representing the
     *			data held on the server.
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.4"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.4</A>
     */
    public Record getRecord(int i) {
	InformationFragment frag[] = null; // assign to avoid warning
	try {
	    // Option names here mirror those in the Net::Z3950 Perl module.
	    frag = ifs.getFragment(i+1, 1, new RecordFormatSpecification
		(option("preferredRecordSyntax"),	// record syntax
		 null,					// schema
		 option("elementSetName")));		// element set
	} catch (PresentException ex) {
	    System.err.println(ex);
	    System.exit(1);
	}
	return new Record(frag[0]);
    }

    /**
     * Deletes this ResultSet, informing the server that it may
     * release any resources it has allocated to it.
     *
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.5"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.5</A>
     */
    public void delete() {
	ifs.destroy();
	ifs = null;
    }
}
